@@ -44,14 +44,26 @@ import Data.Certificate.X509.Internal
 import Data.Certificate.X509.Cert
 import Data.Certificate.X509.Ext
 
-data X509 = X509 Certificate (Maybe L.ByteString) (Maybe L.ByteString) SignatureALG [Word8]
-	deriving (Show,Eq)
+data X509 = X509
+	{ x509Cert              :: Certificate          -- ^ the certificate part of a X509 structure
+	, x509CachedSigningData :: (Maybe L.ByteString) -- ^ a cache of the raw representation of the x509 part for signing
+                                                        -- since encoding+decoding might not result in the same data being signed.
+	, x509CachedData        :: (Maybe L.ByteString) -- ^ a cache of the raw representation of the whole x509.
+	, x509SignatureALG      :: SignatureALG         -- ^ the signature algorithm used.
+	, x509Signature         :: [Word8]              -- ^ the signature.
+	} deriving (Show)
+
+instance Eq X509 where
+	x1 == x2 =
+		(x509Cert x1         == x509Cert x2)         &&
+		(x509SignatureALG x1 == x509SignatureALG x2) &&
+		(x509Signature x1    == x509Signature x2)
 
 {- | get signing data related to a X509 message,
  - which is either the cached data or the encoded certificate -}
 getSigningData :: X509 -> L.ByteString
-getSigningData (X509 _    (Just e) _ _ _)   = e
-getSigningData (X509 cert Nothing _ _ _) = e
+getSigningData (X509 _    (Just e) _ _ _) = e
+getSigningData (X509 cert Nothing _ _ _)  = e
 	where
 		(Right e) = encodeASN1Stream header
 		header    = asn1Container Sequence $ encodeCertificateHeader cert
